with Customers;
with Service_Ready;
with Ada.Numerics.Discrete_Random;
with Ada.Text_IO; use Ada.Text_IO;

package body Barber is

   subtype Interval is Integer range 1 .. 5;

   task body Barber is
      --  the barber needs a local instance of a random number generator
      --+ to determine variable duration interval for customer service
      --+ in the interval 1-5
      package Random_Interval is new Ada.Numerics.Discrete_Random (Interval);
      use Random_Interval;
      G : Generator;
      --------------
      --  declaration first
      procedure Service_Time (G : in     Random_Interval.Generator);
      Client : Customers.Counter_T := Customers.Counter_T'First;
      --------------
      --  specification afterwards
      procedure Service_Time (G : in     Random_Interval.Generator) is
      begin
         --  the barber needs no time accuracy, hence
         --+ relative suspension will do
         delay Duration (Random_Interval.Random (G));
      end Service_Time;
      --------------
      use Customers;
   begin
      --  the barber initialises the random number generator
      Reset (G);
      loop
         --  the barber waits for a customer to walk in
         Customers.Wait;
         Client := Client + 1;
         Put_Line ("Barber servicing customer " &
                   Counter_T'Image (Client) & ".");
         --  the barber simulates service by waiting for
         --+ random duration
         Service_Time (G);
         --  and then notifies any other customer in the queue
         --+ that a new service can now initiate
         Service_Ready.Signal;
      end loop;
   end Barber;

end Barber;
